#include "stdafx.h"
#include "PT1SendCtrlCmdUtil.h"


PipeClient::PipeClient()
{
	m_hEventConnect = _CreateEvent(FALSE, FALSE, CMD_PT1_CTRL_EVENT_WAIT_CONNECT );
}

PipeClient::~PipeClient()
{
	CloseHandle(m_hEventConnect);
}

DWORD PipeClient::WaitConnect()
{
	if( m_hEventConnect == NULL ){
		return CMD_ERR;
	}
	DWORD dwRet = CMD_SUCCESS;
	if(WaitForSingleObject(m_hEventConnect, CONNECT_TIMEOUT) != WAIT_OBJECT_0){
		dwRet = CMD_ERR_TIMEOUT;
	}
	
	return dwRet;
}

DWORD PipeClient::Send(HANDLE hPipe, CMD_STREAM &stream)
{
	DWORD dwWrite = 0;
	DWORD dwSendNum = 0;
	DWORD dwSize = stream.GetSize();
	while(dwSendNum < dwSize ){
		DWORD dwSendSize = 0;
		if( dwSize - dwSendNum < SEND_BUFF_SIZE ){
			dwSendSize = dwSize - dwSendNum;
		}else{
			dwSendSize = SEND_BUFF_SIZE;
		}
		if( WriteFile(hPipe, stream.bData+dwSendNum, dwSendSize, &dwWrite, NULL ) == FALSE ){
			return CMD_ERR;
		}
		dwSendNum+=dwWrite;
	}
	return CMD_SUCCESS;
}

DWORD PipeClient::Recv(HANDLE hPipe, CMD_STREAM &stream) {
	DWORD dwRead = 0;
	DWORD dwReadNum = 0;
	DWORD dwSize = stream.GetSize();
	
	stream.SetSize( dwSize );
	while(dwReadNum < dwSize ){
		DWORD dwReadSize = 0;
		if( dwSize - dwReadNum < RES_BUFF_SIZE ){
			dwReadSize = dwSize-dwReadNum;
		}else{
			dwReadSize = RES_BUFF_SIZE;
		}
		if( ReadFile(hPipe, stream.bData+dwReadNum, dwReadSize, &dwRead, NULL ) == FALSE ){
			return CMD_ERR;
		}
		dwReadNum+=dwRead;
	}
	return CMD_SUCCESS;
}

HANDLE PipeClient::ConnectPipe(LPCWSTR lpwszPipeName )
{
	DWORD dwRet = WaitConnect();
	if( dwRet != CMD_SUCCESS ){
		return INVALID_HANDLE_VALUE;
	}

	HANDLE hPipe;
	int retry = 2;
	do {
		hPipe = _CreateFile(lpwszPipeName, GENERIC_READ|GENERIC_WRITE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if( hPipe != INVALID_HANDLE_VALUE ){
			break;
		} 
		DWORD lastError = GetLastError();
		Sleep(100);
		switch( lastError ) {
			case ERROR_PIPE_BUSY:
				if( WaitNamedPipe( lpwszPipeName, 5000 ) == 0 ) {
					return INVALID_HANDLE_VALUE;
				}
				break;
			default:
				break;
		}
		--retry;
	} while( retry > 0 );
	if( retry == 0 ) {
		return INVALID_HANDLE_VALUE;
	}
	return hPipe;
}

DWORD PipeClient::SendDefCmd(LPCWSTR lpwszPipeName, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return CMD_ERR_INVALID_ARG;
	}

	HANDLE hPipe = ConnectPipe(lpwszPipeName);
	if( hPipe == INVALID_HANDLE_VALUE ) {
		return CMD_ERR_CONNECT;
	}

	DWORD dwWrite = 0;
	DWORD dwRead = 0;
	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		CloseHandle(hPipe);
		return CMD_ERR;
	}
	if( pstSend->GetSize() > 0 ){
		if( pstSend->bData == NULL ){
			CloseHandle(hPipe);
			return CMD_ERR_INVALID_ARG;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle(hPipe);
			return dwRet;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		CloseHandle(hPipe);
		return CMD_ERR;
	}
	if( pstRes->GetSize() > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle(hPipe);
			return dwRet;
		}
	}
	CloseHandle(hPipe);

	return pstRes->dwParam;
}

HANDLE PipeClient::SendEnumCmdFirst(LPCWSTR lpwszPipeName, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return INVALID_HANDLE_VALUE;
	}

	HANDLE hPipe = ConnectPipe(lpwszPipeName);
	if( hPipe == INVALID_HANDLE_VALUE ) {
		return hPipe;
	}
	DWORD dwWrite = 0;
	DWORD dwRead = 0;

	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		CloseHandle(hPipe);
		return INVALID_HANDLE_VALUE;
	}
	if( pstSend->GetSize() > 0 ){
		if( pstSend->bData == NULL ){
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		CloseHandle( hPipe );
		return INVALID_HANDLE_VALUE;
	}
	if( pstRes->GetSize() > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
	}
	return hPipe;
}

DWORD PipeClient::SendEnumCmdNext(HANDLE hPipe, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return CMD_ERR_INVALID_ARG;
	}

	DWORD dwWrite = 0;
	DWORD dwRead = 0;

	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstSend->GetSize() > 0 ){
		if( pstSend->bData == NULL ){
			return CMD_ERR_INVALID_ARG;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstRes->GetSize() > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}

	return pstRes->dwParam;
}

DWORD PipeClient::SendCloseExe()
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_CLOSE_EXE;

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendOpenTuner(BOOL bSate, int* piID, DWORD *pUnique, HANDLE *phFileMapping)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_OPEN_TUNER;
	CreateDefStream2(bSate, GetCurrentProcessId(), &stSend);

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData3((DWORD*)piID, pUnique, (DWORD *)phFileMapping, stRes.bData);
	}

	return dwRet;
}

DWORD PipeClient::SendOpenTuner2(BOOL bSate, int iTunerID, int* piID, DWORD *pUnique, HANDLE *phFileMapping)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_OPEN_TUNER2;
	CreateDefStream3(bSate, iTunerID, GetCurrentProcessId(), &stSend);

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData3((DWORD*)piID, pUnique, (DWORD *)phFileMapping, stRes.bData);
	}

	return dwRet;
}

DWORD PipeClient::SendCloseTuner(int iID)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_CLOSE_TUNER;
	CreateDefStream(iID, &stSend);

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendSetCh(int iID, DWORD dwCh, DWORD dwTSID)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_SET_CH;
	CreateDefStream3(iID, dwCh, dwTSID, &stSend);

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendGetSignal(int iID, DWORD* pdwCn100)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_GET_SIGNAL;
	CreateDefStream(iID, &stSend);

	DWORD dwRet = SendDefCmd( CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData(pdwCn100, stRes.bData);
	}

	return dwRet;
}

DWORD CMemClient::Open(int iID, DWORD unique, HANDLE hFileMapping )
{
	wstring serverName;
	Format(serverName, L"%s%08x%08x", CMD_PT1_DATA, iID, unique );

	DWORD dwRet = CMD_SUCCESS;

	wstring eventCallName;
	Format( eventCallName, L"Global\\%sEventCall", serverName.c_str() );
	m_hEventCall = _CreateEvent(TRUE, FALSE, eventCallName.c_str());

	wstring eventReturnName;
	Format( eventReturnName, L"Global\\%sEventReturn", serverName.c_str() );
	m_hEventReturn = _CreateEvent(TRUE, FALSE, eventReturnName.c_str());

	m_hMapping = hFileMapping;
	m_buf = (BYTE *)MapViewOfFile( m_hMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
	return dwRet;
}
void CMemClient::Close() 
{
	UnmapViewOfFile( m_buf );
	CloseHandle( m_hMapping );
	CloseHandle( m_hEventReturn );
	CloseHandle( m_hEventCall );
}

DWORD CMemClient::GetTSBlock()
{
	*(DWORD *)m_buf = 0;
	SetEvent( m_hEventCall );
	DWORD dwRes = WaitForSingleObject( m_hEventReturn, 2000);
	
	if( dwRes == WAIT_OBJECT_0 ) {
		return *(DWORD *)m_buf;
	} else {
		return 0;
	}
}

BYTE *CMemClient::GetBufferPointer()
{
	return m_buf;
}


