#include "StdAfx.h"
#include "PipeServer.h"
#include <process.h>

#define PIPE_TIMEOUT 500

CPipeServer::CPipeServer(void)
{
	m_pCmdProc = NULL;
	m_pParam = NULL;
	m_strPipeName = L"";

	m_hPipe = NULL;
	m_hThread = NULL;
	m_bStop = false;
	InitializeCriticalSection( &m_cs );
}

CPipeServer::~CPipeServer(void)
{
	StopServer();
	DeleteCriticalSection( &m_cs );
}

void CPipeServer::Lock() {
	EnterCriticalSection( &m_cs );	
}

void CPipeServer::Unlock() {
	LeaveCriticalSection( &m_cs );
}

BOOL CPipeServer::StartServer(LPCWSTR lpszEventConnectName, LPCWSTR lpwszPipeName, CMD_CALLBACK_PROC pfnCmdProc, void* pParam, int iThreadPriority)
{
	if( pfnCmdProc == NULL || pParam == NULL || lpwszPipeName == NULL ){
		return FALSE;
	}
	if( m_hThread != NULL ){
		return FALSE;
	}
	m_pCmdProc = pfnCmdProc;
	m_pParam = pParam;
	m_strPipeName = lpwszPipeName;
	m_iThreadPriority = iThreadPriority;

	m_hPipe = _CreateNamedPipe(m_strPipeName.c_str(), PIPE_ACCESS_DUPLEX | FILE_FLAG_WRITE_THROUGH | FILE_FLAG_OVERLAPPED,
		PIPE_TYPE_BYTE, 1,
		SEND_BUFF_SIZE, RES_BUFF_SIZE, PIPE_TIMEOUT);

	m_hStopEvent = _CreateEvent(FALSE, FALSE, NULL);
	m_hEventConnect = _CreateEvent(FALSE, FALSE, lpszEventConnectName );

	m_hThread = (HANDLE)_beginthreadex(NULL, 0, ServerThread, (LPVOID)this, CREATE_SUSPENDED, NULL);
	ResumeThread(m_hThread);

	return TRUE;
}

void CPipeServer::StopServer()
{
	if( m_hThread != NULL ){
		m_bStop = true;
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 12000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	
	if (m_hPipe != NULL) {
		FlushFileBuffers(m_hPipe);
		DisconnectNamedPipe(m_hPipe);
		CloseHandle(m_hPipe);
		m_hPipe = NULL;
	}
	if( m_hStopEvent != NULL ) {
		CloseHandle(m_hStopEvent);
		m_hStopEvent = NULL;
	}	
	if( m_hEventConnect != NULL ) {
		CloseHandle(m_hEventConnect);
		m_hEventConnect = NULL;
	}	

}

UINT WINAPI CPipeServer::ServerThread(LPVOID pParam)
{
	CPipeServer* pSys = (CPipeServer*)pParam;
	return pSys->ServerThread();
}

UINT WINAPI CPipeServer::ServerThread()
{

	HANDLE hCurThread = GetCurrentThread();
	SetThreadPriority(hCurThread, m_iThreadPriority);

	HANDLE hEventArray[2];
	OVERLAPPED stOver;

	hEventArray[0] = m_hStopEvent;
	hEventArray[1] = _CreateEvent(FALSE, FALSE, NULL);

	ZeroMemory(&stOver, sizeof(OVERLAPPED));
	stOver.hEvent = hEventArray[1];
	ConnectNamedPipe(m_hPipe, &stOver );

	while(1){
		Lock();
		if( m_bStop == true ) break;
		Unlock();

		SetEvent( m_hEventConnect );
		DWORD dwRes = WaitForMultipleObjects(2, hEventArray, FALSE, 10 * 1000 );
		if( dwRes == WAIT_OBJECT_0 ){
			//STOP
			ResetEvent( m_hEventConnect );
			break;
		}else if( dwRes == WAIT_OBJECT_0+1 ){
			Lock();
			//R}hM
			if( m_pCmdProc != NULL){
				CMD_STREAM stCmd;
				CMD_STREAM stRes;
				DWORD dwRead = 0;
				DWORD dwWrite = 0;
				do{
					if( ReadFile(m_hPipe, &stCmd, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
						break;
					}
					if( stCmd.GetSize() > 0 ){
						stCmd.SetSize( stCmd.GetSize() );
						DWORD dwReadNum = 0;
						while(dwReadNum < stCmd.GetSize() ){
							DWORD dwReadSize = 0;
							if( stCmd.GetSize()-dwReadNum < SEND_BUFF_SIZE ){
								dwReadSize = stCmd.GetSize()-dwReadNum;
							}else{
								dwReadSize = SEND_BUFF_SIZE;
							}
							if( ReadFile(m_hPipe, stCmd.bData+dwReadNum, dwReadSize, &dwRead, NULL ) == FALSE ){
								break;
							}
							dwReadNum+=dwRead;
						}
						if( dwReadNum < stCmd.GetSize() ){
							break;
						}
					}

					m_pCmdProc(m_pParam, &stCmd, &stRes);

					if( WriteFile(m_hPipe, &stRes, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
						break;
					}
					if( stRes.GetSize() > 0 ){
						if( stRes.bData == NULL ){
							break;
						}
						DWORD dwSendNum = 0;
						while(dwSendNum < stRes.GetSize() ){
							DWORD dwSendSize = 0;
							if( stRes.GetSize()-dwSendNum < RES_BUFF_SIZE ){
								dwSendSize = stRes.GetSize()-dwSendNum;
							}else{
								dwSendSize = RES_BUFF_SIZE;
							}
							if( WriteFile(m_hPipe, stRes.bData+dwSendNum, dwSendSize, &dwWrite, NULL ) == FALSE ){
								break;
							}
							dwSendNum+=dwWrite;
						}
						if( dwSendNum < stRes.GetSize() ){
							break;
						}
					}
				}while(stRes.dwParam == CMD_NEXT); //Emunp̌JԂ
			}
			FlushFileBuffers(m_hPipe);
			DisconnectNamedPipe(m_hPipe);

			ZeroMemory(&stOver, sizeof(OVERLAPPED));
			stOver.hEvent = hEventArray[1];
			ConnectNamedPipe(m_hPipe, &stOver );
			Unlock();
		} else if( dwRes == WAIT_TIMEOUT ) {
			//SetEvent( m_hEventConnect );
		}
	}

	CloseHandle(hEventArray[1]);

	return 0;
}


