#include "stdafx.h"
#include "Util.h"

HANDLE _CreateEvent(BOOL bManualReset, BOOL bInitialState, LPCTSTR lpName)
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateEvent(&sa, bManualReset, bInitialState, lpName);
}

HANDLE _CreateFile( LPCTSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateFile( lpFileName, dwDesiredAccess, dwShareMode, &sa, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile );
}

HANDLE _CreateMutex( BOOL bInitialOwner, LPCTSTR lpName )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateMutex( &sa, bInitialOwner, lpName );
}

HANDLE _CreateFileMapping( HANDLE hFile, DWORD flProtect, DWORD dwMaximumSizeHigh, DWORD dwMaximumSizeLow, LPCTSTR lpName)
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateFileMapping( hFile, &sa, flProtect, dwMaximumSizeHigh, dwMaximumSizeLow, lpName);
}

HANDLE _CreateNamedPipe( LPCTSTR lpName, DWORD dwOpenMode, DWORD dwPipeMode, DWORD nMaxInstances, DWORD nOutBufferSize, DWORD nInBufferSize, DWORD nDefaultTimeOut )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateNamedPipe( lpName, dwOpenMode, dwPipeMode, nMaxInstances, nOutBufferSize, nInBufferSize, nDefaultTimeOut, &sa );
}

BOOL _CreateDirectory( LPCTSTR lpPathName )
{
	BOOL bRet = FALSE;
	if( _tcslen(lpPathName) > 2 ){
		TCHAR szCreatePath[MAX_PATH+1] = _T("");
		szCreatePath[0] = lpPathName[0];
		szCreatePath[1] = lpPathName[1];
		
		for (int i = 2; i < (int)_tcslen(lpPathName); i++) {
			szCreatePath[i] = lpPathName[i];
			if (szCreatePath[i] == '\\') {
				szCreatePath[i+1] = '\0';
				if ( GetFileAttributes(szCreatePath) == 0xFFFFFFFF ) {
					bRet = ::CreateDirectory( szCreatePath, NULL );
				}
			}
		}
		if ( GetFileAttributes(szCreatePath) == 0xFFFFFFFF ) {
			bRet = ::CreateDirectory( szCreatePath, NULL );
		}
	}

	return bRet;
}

HANDLE _CreateFile2( LPCTSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile )
{
	HANDLE hFile =  ::CreateFile( lpFileName, dwDesiredAccess, dwShareMode, NULL, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile );
	if( hFile == INVALID_HANDLE_VALUE ){
		TCHAR* p = (TCHAR*)_tcsrchr(lpFileName, '\\');
		TCHAR* szDirPath = NULL;
		if( p != NULL ){
			int iSize = (int)(p - lpFileName);
			szDirPath = new TCHAR[iSize+1];
			_tcsncpy_s(szDirPath, iSize+1, lpFileName, iSize);
		}
		if( szDirPath != NULL ){
			_CreateDirectory(szDirPath);
			delete[] szDirPath;
			hFile =  ::CreateFile( lpFileName, dwDesiredAccess, dwShareMode, NULL, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile );
		}
	}
	return hFile;
}


class CS {
	CRITICAL_SECTION criticalSection;
public:
	CS() {
		InitializeCriticalSection( &criticalSection );
	}
	~CS() {
		DeleteCriticalSection( &criticalSection );
	}
	void Enter() {
		EnterCriticalSection( &criticalSection );
	}
	void Leave() {
		LeaveCriticalSection( &criticalSection );
	}
} cs;

typedef std::vector<BYTE *> BufList;
typedef std::map< int, std::vector<BYTE *> > BufListMap;
BufListMap bufListMap;

BYTE *NEW_DATA_BUFF(int size )
{
	_ASSERT( size > 0 );
	//WCHAR str[64];
	//wsprintf( str, L"NEW_DATA_BUFF size=%d\n", size );
	//OutputDebugString( str );

	cs.Enter();
	BYTE *ret;
	BufList &bufList = bufListMap[size];
	if( bufList.size() > 0 ) {
		ret = bufList[bufList.size()-1];
		bufList.pop_back();
	} else {
		ret = new BYTE[size];
	}
	cs.Leave();
	return ret;
}

void DELETE_DATA_BUFF(BYTE *&buf, int size)
{
	_ASSERT( size > 0 );
	if( buf == NULL ) return;

	//WCHAR str[64];
	//wsprintf( str, L"DELETE_DATA_BUFF size=%d\n", size );
	//OutputDebugString( str );

	cs.Enter();
	BufList &bufList = bufListMap[size];

	bufList.push_back( buf );
	if( bufList.size() > 64 ) {
		delete bufList[bufList.size()-1];
		bufList.pop_back();
	}
	buf = NULL;
	cs.Leave();
}

void PURGE_DATA_BUFF()
{
	cs.Enter();
	for( BufListMap::iterator i = bufListMap.begin(); i != bufListMap.end(); ++i ) {
		BufList &list = (*i).second;
		//WCHAR str[64];
		//wsprintf( str, L"Purge size=%d count=%d\n", (*i).first, list.size() );
		//OutputDebugString( str );
		for( BufList::iterator j = list.begin(); j != list.end(); ++j ) {
			delete[] *j;
			*j = NULL;
		}
	}
	cs.Leave();
}