#include "StdAfx.h"
#include "DataIO.h"
#include <process.h>

#define MAX_DATA_BUFF_COUNT 500

CDataIO::CDataIO(void)
{
	VIRTUAL_COUNT = 8;

	m_hStopEvent = _CreateEvent(FALSE, FALSE, NULL);
	m_hThread = NULL;

	m_pcDevice = NULL;

	m_bQuit = false;

	for(int j = 0; j < 4; j++ ) {
		dataIoTS[j] = NULL; 
		m_CurrentWriteBlock[j] = new TS_BLOCK;
		m_rawPacketCur[j] = 0;
	}
	m_bDMABuff = (BYTE *)_aligned_malloc( READ_BLOCK_SIZE, 64 );
	m_rawPacket = (DWORD (*)[4][64]) _aligned_malloc(4*256,16);

	InitializeCriticalSection( &csDispatch );
}

CDataIO::~CDataIO(void)
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	if( m_hStopEvent != NULL ){
		::CloseHandle(m_hStopEvent);
		m_hStopEvent = NULL;
	}

	for(int j = 0; j < 4; j++ ) {
		CDataIO::DATAIO_TS *ts = dataIoTS[j];
		if(ts) delete ts;
		SAFE_DELETE(m_CurrentWriteBlock[j]);
	}
	_aligned_free( m_bDMABuff );

	_aligned_free( m_rawPacket );

	DeleteCriticalSection( &csDispatch );
}

void CDataIO::ClearBuff(int iID)
{
	//int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner ); 
	if ( dataIoTS[id] )
		dataIoTS[id]->ClearBlockBuffer();

	m_CurrentWriteBlock[id]->dwSetSize = 0;
}

void CDataIO::DATAIO_TS::ClearBlockBuffer()
{
	Lock();
	m_dwOverFlowCount = 0;
	while ( !m_Buff.empty() ) {
		SAFE_DELETE( m_Buff.front() );
		m_Buff.pop();
	}
	UnLock();
}

void CDataIO::Run()
{
	if( m_hThread != NULL ){
		return ;
	}
	status enStatus;

	bool bEnalbe = true;
	enStatus = m_pcDevice->GetTransferEnable(&bEnalbe);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	if( bEnalbe == true ){
		enStatus = m_pcDevice->SetTransferEnable(false);
		if( enStatus != PT::STATUS_OK ){
			return ;
		}
	}

	enStatus = m_pcDevice->SetBufferInfo(NULL);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	PT::Device::BufferInfo bufferInfo;
	bufferInfo.VirtualSize  = VIRTUAL_IMAGE_COUNT;
	bufferInfo.VirtualCount = VIRTUAL_COUNT;
	bufferInfo.LockSize     = LOCK_SIZE;
	enStatus = m_pcDevice->SetBufferInfo(&bufferInfo);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	ResetDMA();

	::ResetEvent(m_hStopEvent);
	m_bQuit = false;
	m_hThread = (HANDLE)_beginthreadex(NULL, 0, RecvThread, (LPVOID)this, CREATE_SUSPENDED, NULL);
	SetThreadPriority( m_hThread, THREAD_PRIORITY_ABOVE_NORMAL );
	ResumeThread(m_hThread);
}

void CDataIO::ResetDMA()
{
	status enStatus;
	//OutputDebugString( L"ResetDMA\n" );


	// DMA ]~

	bool bEnalbe = true;
	enStatus = m_pcDevice->GetTransferEnable(&bEnalbe);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	if( bEnalbe == true ){
		enStatus = m_pcDevice->SetTransferEnable(false);
		if( enStatus != PT::STATUS_OK ){
			return ;
		}
	}

	mVirtualIndex=0;
	mImageIndex=0;
	mBlockIndex=0;

	// DMA ]ǂ܂Ői񂾂𒲂ׂ邽߁AeubN̖ 0 ŃNA
	for (uint i=0; i<VIRTUAL_COUNT; i++) {
		for (uint j=0; j<VIRTUAL_IMAGE_COUNT; j++) {
			for (uint k=0; k<READ_BLOCK_COUNT; k++) {
				Clear(i, j, k);
			}
		}
	}

	// ]JE^Zbg
	enStatus = m_pcDevice->ResetTransferCounter();
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	// ]JE^CNg
	for (uint i=0; i<VIRTUAL_IMAGE_COUNT*VIRTUAL_COUNT; i++) {
		enStatus = m_pcDevice->IncrementTransferCounter();
		if( enStatus != PT::STATUS_OK ){
			return ;
		}
	}

	//TSobt@SĔj
	for( int i=0; i<4; ++i ) {
		if( dataIoTS[i] ) {
			dataIoTS[i]->ClearBlockBuffer();
		}
	}
	//}CNpPbgJ[\NA
	for( int i=0; i<4; ++i ) {
		m_rawPacketCur[i] = 0;
	}

	Sleep(20);

	// DMA ]
	enStatus = m_pcDevice->SetTransferEnable(true);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
}

void CDataIO::Stop()
{
	if( m_hThread != NULL ){
		m_bQuit = true;
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}

	status enStatus;

	bool bEnalbe = true;
	enStatus = m_pcDevice->GetTransferEnable(&bEnalbe);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	if( bEnalbe == true ){
		enStatus = m_pcDevice->SetTransferEnable(false);
		if( enStatus != PT::STATUS_OK ){
			return ;
		}
	}
}

BOOL CDataIO::EnableTuner(int iID, BOOL bEnable, DWORD unique)
{
	BOOL bRet = TRUE;

	//int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner ); 
	DATAIO_TS *&ts = dataIoTS[id];

	//static const int IDToPacketId[] = { 1, 3, 0, 2};

	EnterCriticalSection( &csDispatch );

	m_CurrentWriteBlock[id]->dwSetSize = 0;
	m_rawPacketCur[ id ] = 0;


	if( bEnable == TRUE ){
		if( !ts ) ts = new DATAIO_TS( id, iID, unique );
		LeaveCriticalSection( &csDispatch );

		bRet = ts->Start();
		if( bRet == FALSE ) {
			delete ts;
			ts = NULL;
		}
	} else {
		LeaveCriticalSection( &csDispatch );

		//Stopsꍇ^CAEg(2b)ԏ߂ĂȂ
		//NeBJZbVŎs킯ɂȂ
		if( ts ) {
			ts->Stop();
			EnterCriticalSection( &csDispatch );
			delete ts;
			LeaveCriticalSection( &csDispatch );
			ts = NULL;
		}

	}
	return bRet;
}

UINT WINAPI CDataIO::RecvThread(LPVOID pParam)
{
	CDataIO* pSys = (CDataIO*)pParam;

	HANDLE hCurThread = GetCurrentThread();
	SetThreadPriority(hCurThread, THREAD_PRIORITY_HIGHEST);

	pSys->mVirtualIndex = 0;
	pSys->mImageIndex = 0;
	pSys->mBlockIndex = 0;

	while (true) {
		DWORD dwRes = WaitForSingleObject(pSys->m_hStopEvent, 0);
		if( dwRes == WAIT_OBJECT_0 ){
			//STOP
			break;
		}

		bool b;
		
		b = pSys->WaitBlock();
		if (b == false) break;

		pSys->CopyBlock();

		EnterCriticalSection( &pSys->csDispatch );
		b = pSys->DispatchBlock();
		LeaveCriticalSection( &pSys->csDispatch );
		if (b == false) break;
	}

	return 0;
}

// 1ubN DMA ]I邩 mQuit  true ɂȂ܂ő҂
bool CDataIO::WaitBlock()
{
	bool b = true;

	while (true) {
		if (m_bQuit) {
			b = false;
			break;
		}

		// ubN̖ 0 łȂ΁ÃubN DMA ]ƂɂȂ
		if (Read(mVirtualIndex, mImageIndex, mBlockIndex) != 0) break;
		Sleep(5);
	}
	//::wprintf(L"(mVirtualIndex, mImageIndex, mBlockIndex) = (%d, %d, %d) ̓]I܂B\n", mVirtualIndex, mImageIndex, mBlockIndex);

	return b;
}

// 1ubÑf[^e|̈ɃRs[BCPU 猩 DMA obt@̓LbVȂ߁A
// LbV̈ɃRs[ĂANZXƌ܂܂B
void CDataIO::CopyBlock()
{
	//OutputDebugString( L"CopyBlock\n" );
	status enStatus;

	void *voidPtr;
	enStatus = m_pcDevice->GetBufferPtr(mVirtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	DWORD dwOffset = ((TRANSFER_SIZE*mImageIndex) + (READ_BLOCK_SIZE*mBlockIndex));

	memcpy(m_bDMABuff, (BYTE*)voidPtr+dwOffset, READ_BLOCK_SIZE);

	// Rs[ÎŁAubN̖ 0 ɂ܂B
	uint *ptr = static_cast<uint *>(voidPtr);
	ptr[Offset(mImageIndex, mBlockIndex, READ_BLOCK_SIZE)-1] = 0;

	if (READ_BLOCK_COUNT <= ++mBlockIndex) {
		mBlockIndex = 0;

		// ]JE^ OS::Memory::PAGE_SIZE * PT::Device::BUFFER_PAGE_COUNT oCgƂɃCNg܂B
		enStatus = m_pcDevice->IncrementTransferCounter();
		if( enStatus != PT::STATUS_OK ){
			return ;
		}

		if (VIRTUAL_IMAGE_COUNT <= ++mImageIndex) {
			mImageIndex = 0;
			if (VIRTUAL_COUNT <= ++mVirtualIndex) {
				mVirtualIndex = 0;
			}
		}
	}
}

void CDataIO::Clear(uint virtualIndex, uint imageIndex, uint blockIndex)
{
	void *voidPtr;
	status enStatus = m_pcDevice->GetBufferPtr(virtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	uint *ptr = static_cast<uint *>(voidPtr);
	ptr[Offset(imageIndex, blockIndex, READ_BLOCK_SIZE)-1] = 0;
}

uint CDataIO::Read(uint virtualIndex, uint imageIndex, uint blockIndex) const
{
	void *voidPtr;
	status enStatus = m_pcDevice->GetBufferPtr(virtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return 0;
	}

	volatile const uint *ptr = static_cast<volatile const uint *>(voidPtr);
	return ptr[Offset(imageIndex, blockIndex, READ_BLOCK_SIZE)-1];
}

uint CDataIO::Offset(uint imageIndex, uint blockIndex, uint additionalOffset) const
{
	uint offset = ((TRANSFER_SIZE*imageIndex) + (READ_BLOCK_SIZE*blockIndex) + additionalOffset) / sizeof(uint);

	return offset;
}

bool CDataIO::DispatchBlock()
{
	//OutputDebugString( L"DispatchBlock\n" );
	static const int packetIdToID[] = { 2, 0, 3, 1};

	const DWORD *ptr = (DWORD*)m_bDMABuff;

	for (uint i=0; i<READ_BLOCK_SIZE / 4; ++i) {
		DWORD data = *ptr++;
		uint packetStart = BIT_SHIFT_MASK( data, 25, 1);
		uint packetId = BIT_SHIFT_MASK(data, 29,  3) - 1;
		if( packetId >= 4 ) continue;

		int id = packetIdToID[packetId];
		if( packetStart ) {
			//if( m_rawPacketCur[id] != 0 ) error;
			m_rawPacketCur[id] = 0;
		}
		(*m_rawPacket)[id][ m_rawPacketCur[id]++ ] = data;
		if( m_rawPacketCur[id] == 189/3 ) {
			m_rawPacketCur[id] = 0;
			if( MicroPacket( id ) == false ) {
				DispError();
				break;
			}
		}
	}

	return true;
}

//słꍇtrue słȂꍇfalse
bool CDataIO::MicroPacket(DWORD id)
{

	BYTE *data = (BYTE *)(*m_rawPacket)[id];
	BYTE *dest = m_CurrentWriteBlock[id]->aBuff + m_CurrentWriteBlock[id]->dwSetSize;
	//rawPacketTSpPbgւ̕ϊ
	for( int i=0; i< 62; ++i ) { // 3*62 =186
		dest[0] = data[2];
		dest[1] = data[1];
		dest[2] = data[0];
		data += 4;
		dest += 3;
	}
	*dest++ = data[2];
	*dest++ = data[1];
	uint dmaError = BIT_SHIFT_MASK( data[3], 0, 1);

	if( dmaError ) {
		OutputDebugString( _T( "dmaError" ) );
		return false;
	}
	if( *(m_CurrentWriteBlock[id]->aBuff + m_CurrentWriteBlock[id]->dwSetSize) != 0x47 ) {
		//TSpPbgƂĔF߂Ȃ
		OutputDebugString( _T( "TSpPbgƂĔF߂Ȃ" ) );
		return true;
	}

	m_CurrentWriteBlock[id]->dwSetSize+=188;
	if( m_CurrentWriteBlock[id]->dwSetSize >= TS_BLOCK_SIZE ){
		if(dataIoTS[id]) {
			if( dataIoTS[id]->PushBlock( m_CurrentWriteBlock[id] ) ) {
				m_CurrentWriteBlock[id] = new TS_BLOCK;
			}
		}
		m_CurrentWriteBlock[id]->dwSetSize = 0;
	}
	return true;
}

void CDataIO::DispError() {
	// G[̌𒲂ׂ
	PT::Device::TransferInfo info;
	status enStatus = m_pcDevice->GetTransferInfo(&info);
	if( enStatus == PT::STATUS_OK ){
		if (info.TransferCounter0) {
			OutputDebugString(L"]JE^ 0 ł̂oB");
			ResetDMA();
			return;
		} else if (info.TransferCounter1) {
			OutputDebugString(L"]JE^ 1 ȉɂȂ܂B");
			ResetDMA();
			return;
		} else if (info.BufferOverflow) {
			OutputDebugString(L"PCI oX𒷊ɓnmۂłȂ߁A{[h FIFO ܂B");
			ResetDMA();
			return;
		} else {
			OutputDebugString(L"]G[܂B");
			return;
		}
	}else{
		OutputDebugString(L"GetTransferInfo() err");
		return;
	}
}

DWORD CDataIO::GetOverFlowCount(int iID)
{
	//int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner );
	DWORD dwRet = dataIoTS[id]->GetOverFlowCount();

	return dwRet;
}

HANDLE CDataIO::DuplicateFileMappingHandle(int iID, DWORD processID )
{
	//int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner );
	HANDLE handle = dataIoTS[id]->DuplicateFileMappingHandle( processID );

	return handle;
}


CDataIO::DATAIO_TS::DATAIO_TS( int id, int iID, DWORD unique )
{
	m_dwOverFlowCount = 0;
	m_id = id;

	wstring strName = L"";
	Format(strName, L"%s%08x%08x", CMD_PT1_DATA, iID, unique );
	m_cMem = new CMemServer( strName.c_str() );

	InitializeCriticalSection( &m_csBuff );
}

CDataIO::DATAIO_TS::~DATAIO_TS()
{
	delete m_cMem;

	while( !m_Buff.empty() ) {
		delete m_Buff.front();
		m_Buff.pop();
	}
	DeleteCriticalSection( &m_csBuff );
}

int CDataIO::DATAIO_TS::PopBlock( void *buf, size_t size ) {
	Lock();
	size_t count = m_Buff.size();
	count = min( count, size );
	BYTE *p = (BYTE *)buf;
	for(size_t i=0;i<count; ++i ) {
		TS_BLOCK *data = m_Buff.front();
		m_Buff.pop();
		memcpy( p, data->aBuff, TS_BLOCK_SIZE );
		p += TS_BLOCK_SIZE;
		delete data;
	}
	UnLock();
	return count;
}

bool CDataIO::DATAIO_TS::PushBlock( TS_BLOCK *data)
{
	bool bRet = true;
	Lock();

	if( m_Buff.size() > MAX_DATA_BUFF_COUNT ){
		data->dwSetSize = 0;

		m_dwOverFlowCount++;
		WCHAR str[64];
		wsprintf( str, L"Buff Full ID%02d Count=%d", m_id, m_dwOverFlowCount );
		OutputDebugString( str );
		bRet = false;
	}else{
		m_Buff.push( data );

		m_dwOverFlowCount = 0;
		bRet = true;
	}
	UnLock();
	return bRet;
}

int CALLBACK CDataIO::DATAIO_TS::OutsideCallback(void *pParam, void *buf )
{
	DATAIO_TS *ts = (DATAIO_TS *)pParam;

	int count = ts->PopBlock((BYTE *)buf + 64, 8 );
	*(DWORD *)buf = count;
	return 0;
}

BOOL CDataIO::DATAIO_TS::Start() 
{
	return m_cMem->Start( OutsideCallback, (void *)this, THREAD_PRIORITY_ABOVE_NORMAL);
}

void CDataIO::DATAIO_TS::Stop() 
{
	m_cMem->Stop();
}

HANDLE CDataIO::DATAIO_TS::DuplicateFileMappingHandle( DWORD processID )
{
	return m_cMem->DuplicateFileMappingHandle( processID );
}

DWORD CDataIO::DATAIO_TS::GetOverFlowCount() 
{
	return m_dwOverFlowCount;
}