#pragma once

#include "inc/EARTH_PT.h"
#include "inc/Prefix.h"
#include "../../Common/PT1OutsideCtrlCmdDef.h"
#include "../../Common/PipeServer.h"
#include "../../Common/MemServer.h"

#define TRANSFER_SIZE (4096*PT::Device::BUFFER_PAGE_COUNT)
#define VIRTUAL_IMAGE_COUNT 4
#define LOCK_SIZE 4
#define READ_BLOCK_COUNT 8
#define READ_BLOCK_SIZE (TRANSFER_SIZE / READ_BLOCK_COUNT)

using namespace EARTH;

class CDataIO
{
public:
	CDataIO(void);
	~CDataIO(void);

	void SetDevice(PT::Device* pcDevice){ m_pcDevice = pcDevice; };
	void SetVirtualCount(UINT uiVirtualCount){ VIRTUAL_COUNT = uiVirtualCount; };
	void Run();
	void Stop();
	BOOL EnableTuner(int iID, BOOL bEnable, DWORD unique);
	void ClearBuff(int iID);

	DWORD GetOverFlowCount(int iID);
	HANDLE DuplicateFileMappingHandle(int iID, DWORD processID);

protected:
	UINT VIRTUAL_COUNT;
	PT::Device* m_pcDevice;

	uint mVirtualIndex;
	uint mImageIndex;
	uint mBlockIndex;
	volatile bool m_bQuit;

	HANDLE m_hStopEvent;
	HANDLE m_hThread;
	CRITICAL_SECTION csDispatch;

	struct TS_BLOCK {
		DWORD dwSetSize;
		BYTE aBuff[TS_BLOCK_SIZE];
		TS_BLOCK(void){
			dwSetSize = 0;
		}
		~TS_BLOCK(void){
		}
		TS_BLOCK(const TS_BLOCK &);

		void *operator new(size_t n ) {
			return NEW_DATA_BUFF( n );
		}
		void operator delete(void *p) {
			BYTE *b = (BYTE *)p;
			DELETE_DATA_BUFF( b, sizeof(TS_BLOCK) );
		}
		size_t GetBufferSize() {
			return TS_BLOCK_SIZE;
		}
	};

	struct DATAIO_TS {
	public:
		DATAIO_TS( int id, int iID, DWORD unique );
		~DATAIO_TS();

		void Lock() {
			EnterCriticalSection( &m_csBuff );
		}
		void UnLock() {
			LeaveCriticalSection( &m_csBuff );
		}
		
		void ClearBlockBuffer();

		int PopBlock( void *buf, size_t size );
		bool PushBlock( TS_BLOCK* data);

		BOOL Start();
		void Stop();
		HANDLE DuplicateFileMappingHandle( DWORD processID );
		DWORD GetOverFlowCount();

	private:
		DATAIO_TS( const DATAIO_TS &);
		static int CALLBACK OutsideCallback(void* pParam, void *buf );

		int m_id;
		DWORD m_dwOverFlowCount;

		CMemServer *m_cMem;
		queue<TS_BLOCK*> m_Buff;
		CRITICAL_SECTION m_csBuff;
	} *dataIoTS[4]; // T0=0 T1=1 S0=2 S1=3

	//`[i[Ă}CNpPbgׂ͂
	//DATAIO_TSɑgݍ܂Ȃ
	DWORD m_rawPacketCur[4];
	DWORD (*m_rawPacket)[4][256/4];

	TS_BLOCK* m_CurrentWriteBlock[4];
	BYTE* m_bDMABuff;

	int GetID(enum PT::Device::ISDB enISDB, uint iTuner ) {
		if( enISDB == PT::Device::ISDB_T ){
			if( iTuner == 0 ){
				return 0;
			}else{
				return 1;
			}
		}else{
			if( iTuner == 0 ){
				return 2;
			}else{
				return 3;
			}
		}
	}

	static UINT WINAPI RecvThread(LPVOID pParam);

	bool WaitBlock();
	void CopyBlock();
	bool DispatchBlock();
	void DispError();
	void Clear(uint virtualIndex, uint imageIndex, uint blockIndex);
	uint Read(uint virtualIndex, uint imageIndex, uint blockIndex) const;
	uint Offset(uint imageIndex, uint blockIndex, uint additionalOffset = 0) const;
	bool MicroPacket(DWORD dwID);

	static int CALLBACK OutsideCmdCallback(void* pParam, CMD_STREAM* pCmdParam, CMD_STREAM* pResParam);

	void ResetDMA();

};
