#include "StdAfx.h"
#include "PT1CtrlMain.h"

extern HANDLE g_hMutex;
extern HANDLE g_hMutex_Exiting;

CPT1CtrlMain::CPT1CtrlMain(void)
{
	m_hStopEvent = _CreateEvent(TRUE, FALSE,NULL);
	m_hAllClose = _CreateEvent(FALSE, FALSE,NULL);
	m_cPipeserver = NULL;
}

CPT1CtrlMain::~CPT1CtrlMain(void)
{
	StopMain();
	if( m_hStopEvent != NULL ){
		CloseHandle(m_hStopEvent);
	}
	if( m_hAllClose != NULL ){
		CloseHandle(m_hAllClose);
	}
}

void CPT1CtrlMain::StartMain()
{
	BOOL bInit = TRUE;
	if( m_cPT1.LoadSDK() == FALSE ){
		OutputDebugString(L"PT SDK̃[hɎs");
		bInit = FALSE;
	}
	if( bInit == TRUE ){
		m_cPT1.Init();
	}

	//PipeT[o[X^[g
	m_cPipeserver = new CPipeServer();
	m_cPipeserver->StartServer( CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, OutsideCmdCallback, this);

	MainLoop();

	if( g_hMutex_Exiting == NULL ) {
		g_hMutex_Exiting = _CreateMutex(TRUE, PT1_CTRL_MUTEX_EXITING);
	}
	if( g_hMutex != NULL ) {
		::CloseHandle(g_hMutex);
		g_hMutex = NULL;
	}

	m_cPipeserver->StopServer();
	delete m_cPipeserver;
	m_cPT1.UnInit();
}

void CPT1CtrlMain::MainLoop()
{
	HANDLE hEvents[2] = { m_hStopEvent, m_hAllClose };
	while(1){
		DWORD result = WaitForMultipleObjects (2, hEvents, FALSE, 15*1000);
		switch( result ) {
			case WAIT_OBJECT_0:
				return;
			case WAIT_OBJECT_0+1:
				Sleep( m_cPT1.GetEndDelay() );
				m_cPipeserver->Lock();
				if( m_cPT1.IsFindOpen() == FALSE ){
					m_cPipeserver->StopServer();
					m_cPipeserver->Unlock();
					return;
				}
				m_cPipeserver->Unlock();
				break;
			case WAIT_TIMEOUT:
				//Avw񂾎p̃`FbN
				if( m_cPT1.CloseChk() == FALSE ){
					SetEvent( m_hAllClose );
					return;
				}
				break;
			default:
				return;
		}
	}
}

void CPT1CtrlMain::StopMain()
{
	if( m_hStopEvent != NULL ){
		SetEvent(m_hStopEvent);
	}
}

int CALLBACK CPT1CtrlMain::OutsideCmdCallback(void* pParam, CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	CPT1CtrlMain* pSys = (CPT1CtrlMain*)pParam;

	switch( pCmdParam->dwParam ){
		case CMD_CLOSE_EXE:
			//OutputDebugString( L"CMD_CLOSE_EXE\n" );
			pSys->CmdCloseExe(pCmdParam, pResParam);
			break;
		case CMD_OPEN_TUNER:
			//OutputDebugString( L"CMD_OPEN_TUNER\n" );
			pSys->CmdOpenTuner(pCmdParam, pResParam);
			break;
		case CMD_CLOSE_TUNER:
			//OutputDebugString( L"CMD_CLOSE_TUNER\n" );
			pSys->CmdCloseTuner(pCmdParam, pResParam);
			break;
		case CMD_SET_CH:
			//OutputDebugString( L"CMD_SET_CH\n" );
			pSys->CmdSetCh(pCmdParam, pResParam);
			break;
		case CMD_GET_SIGNAL:
			//OutputDebugString( L"CMD_GET_SIGNAL\n" );
			pSys->CmdGetSignal(pCmdParam, pResParam);
			break;
		case CMD_OPEN_TUNER2:
			//OutputDebugString( L"CMD_OPEN_TUNER2\n" );
			pSys->CmdOpenTuner2(pCmdParam, pResParam);
			break;
		default:
			//OutputDebugString( L"CMD_NON_SUPPORT\n" );
			pResParam->dwParam = CMD_NON_SUPPORT;
			break;
	}
	return 0;
}

//CMD_CLOSE_EXE PT1Ctrl.exȅI
void CPT1CtrlMain::CmdCloseExe(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	pResParam->dwParam = CMD_SUCCESS;
	StopMain();
}

//CMD_OPEN_TUNER OpenTuner
void CPT1CtrlMain::CmdOpenTuner(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	BOOL bSate;
	DWORD processID;
	DWORD unique = GetTickCount();

	CopyDefData2((DWORD*)&bSate, &processID, pCmdParam->bData);
	int iID = m_cPT1.OpenTuner(bSate, unique);
	if( iID != -1 ){
		HANDLE hDup = m_cPT1.DuplicateFileMappingHandle( iID, processID );
		pResParam->dwParam = CMD_SUCCESS;
		CreateDefStream3(iID, unique, (DWORD)hDup, pResParam);
	}else{
		pResParam->dwParam = CMD_ERR;
	}
}

//CMD_OPEN_TUNER2 OpenTuner2
void CPT1CtrlMain::CmdOpenTuner2(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	BOOL bSate;
	int iTunerID;
	DWORD processID;
	DWORD unique = GetTickCount();

	CopyDefData3((DWORD*)&bSate, (DWORD*)&iTunerID, &processID, pCmdParam->bData);
	int iID = m_cPT1.OpenTuner2(bSate, iTunerID, unique );
	if( iID != -1 ){
		HANDLE hDup = m_cPT1.DuplicateFileMappingHandle( iID, processID );
		pResParam->dwParam = CMD_SUCCESS;
		CreateDefStream3(iID, unique, (DWORD)hDup, pResParam);
	}else{
		pResParam->dwParam = CMD_ERR;
	}
}


//CMD_CLOSE_TUNER CloseTuner
void CPT1CtrlMain::CmdCloseTuner(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	int iID;
	CopyDefData((DWORD*)&iID, pCmdParam->bData);
	m_cPT1.CloseTuner(iID);
	pResParam->dwParam = CMD_SUCCESS;

	if( m_cPT1.IsFindOpen() == FALSE ){
		SetEvent( m_hAllClose );
	}
}

//CMD_SET_CH SetChannel
void CPT1CtrlMain::CmdSetCh(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	int iID;
	DWORD dwCh;
	DWORD dwTSID;
	CopyDefData3((DWORD*)&iID, &dwCh, &dwTSID, pCmdParam->bData);
	if( m_cPT1.SetCh(iID,dwCh,dwTSID) == TRUE ){
		pResParam->dwParam = CMD_SUCCESS;
	}else{
		pResParam->dwParam = CMD_ERR;
	}
}

//CMD_GET_SIGNAL GetSignalLevel
void CPT1CtrlMain::CmdGetSignal(CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	int iID;
	DWORD dwCn100;
	CopyDefData((DWORD*)&iID, pCmdParam->bData);
	dwCn100 = m_cPT1.GetSignal(iID);

	pResParam->dwParam = CMD_SUCCESS;
	CreateDefStream(dwCn100, pResParam);
}
